# Generated by roxygen2: do not edit by hand

S3method(calculate_ratios,PKNCAresults)
S3method(calculate_ratios,data.frame)
S3method(interval_add_impute,PKNCAdata)
S3method(interval_add_impute,data.frame)
S3method(interval_remove_impute,PKNCAdata)
S3method(interval_remove_impute,data.frame)
export(PKNCA_build_units_table)
export(PKNCA_calculate_nca)
export(PKNCA_create_data_object)
export(PKNCA_hl_rules_exclusion)
export(PKNCA_impute_method_start_c1)
export(PKNCA_impute_method_start_logslope)
export(PKNCA_update_data_object)
export(add_f_to_pknca_results)
export(apply_filters)
export(apply_labels)
export(apply_mapping)
export(calculate_f)
export(calculate_ratios)
export(calculate_summary_stats)
export(check_slope_rule_overlap)
export(convert_volume_units)
export(create_metabfl)
export(create_start_impute)
export(detect_study_types)
export(dose_profile_duplicates)
export(export_cdisc)
export(filter_breaks)
export(filter_slopes)
export(flexible_violinboxplot)
export(format_pkncaconc_data)
export(format_pkncadata_intervals)
export(format_pkncadose_data)
export(g_pkcg01_lin)
export(g_pkcg01_log)
export(g_pkcg02_lin)
export(g_pkcg02_log)
export(general_lineplot)
export(general_meanplot)
export(generate_tooltip_text)
export(get_conversion_factor)
export(get_label)
export(interval_add_impute)
export(interval_remove_impute)
export(l_pkcl01)
export(lambda_slope_plot)
export(multiple_matrix_ratios)
export(parse_annotation)
export(pivot_wider_pknca_results)
export(pk.calc.volpk)
export(pk_dose_qc_plot)
export(pkcg01)
export(pkcg02)
export(pknca_calculate_f)
export(read_pk)
export(run_app)
export(simplify_unit)
export(translate_terms)
import(dplyr)
import(formatters)
import(ggplot2)
import(rlistings)
import(tidyr)
importFrom(PKNCA,exclude)
importFrom(PKNCA,pk.calc.c0)
importFrom(dplyr,"%>%")
importFrom(dplyr,`%>%`)
importFrom(dplyr,across)
importFrom(dplyr,add_count)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,rename)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,sym)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(flextable,flextable)
importFrom(formatters,var_labels)
importFrom(ggplot2,aes)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_x_continuous)
importFrom(grid,convertUnit)
importFrom(magrittr,`%>%`)
importFrom(methods,is)
importFrom(officer,add_slide)
importFrom(officer,move_slide)
importFrom(officer,ph_location_type)
importFrom(officer,ph_slidelink)
importFrom(officer,ph_with)
importFrom(officer,read_pptx)
importFrom(plotly,add_trace)
importFrom(plotly,config)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,style)
importFrom(purrr,imap)
importFrom(purrr,pmap)
importFrom(purrr,pmap_chr)
importFrom(rlang,set_names)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(stats,as.formula)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringr,str_glue)
importFrom(stringr,str_split)
importFrom(stringr,str_trim)
importFrom(tern,g_ipp)
importFrom(tidyr,crossing)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unnest)
importFrom(tools,file_ext)
importFrom(units,deparse_unit)
importFrom(units,drop_units)
importFrom(units,set_units)
importFrom(units,units_options)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,read.csv)
